# Disable debug information package creation
%define debug_package %{nil}

# Build mode:
# - upstream
# - downstream [default]
%{expand: %%define build_mode %(echo ${BUILD_MODE:-downstream})}
%if "%{build_mode}" != "downstream" && "%{build_mode}" != "upstream"
        %error BUILD MODE must be downstream or upstream
%endif

# Define the Go Import Path
%global goipath github.com/flightctl/flightctl

Version:        0.3.0
%gometa

Name:           flightctl
Release:        1.20250102151221086407.pr735.193.gc4e86a1%{?dist}
Summary:        Flightctl is a manager of the edge device fleets.

License:        Apache-2.0 AND BSD-2-Clause AND BSD-3-Clause AND ISC AND MIT
URL:            %{gourl}

Source0:        flightctl-0.3.0.tar.gz
%if "%{build_mode}" == "upstream"
    Source1:        %{archivename}-vendor.tar.bz2
%endif

BuildRequires:  git
BuildRequires:  openssl-devel
%if "%{build_mode}" == "upstream"
BuildRequires:  go-rpm-macros
BuildRequires:  compiler(go-compiler)
BuildRequires:  libvirt-devel
%else
BuildRequires: golang
BuildRequires: make
%endif
Requires: openssl

# Skip description for the main package since it won't be created
%description
# Main package is empty and not created.

# cli sub-package
%package cli
Summary: Flightctl CLI
%description cli
Flightctl is a command line interface for managing edge device fleets.

# agent sub-package
%package agent
Summary: Flightctl Agent
%description agent
Flightctl Agent is a component of the flightctl tool.

%prep
%goprep -A
%if "%{build_mode}" == "upstream"
%setup -q -T -D -a1 %{forgesetupargs} -n flightctl-0.3.0
%else
%setup -q %{forgesetupargs}
%endif

%build
    %if "%{build_mode}" == "upstream"
        SOURCE_GIT_TAG=$(git describe --tags --exclude latest)
        SOURCE_GIT_TREE_STATE=$(( ( [ ! -d ".git/" ] || git diff --quiet ) && echo 'clean' ) || echo 'dirty')
        SOURCE_GIT_COMMIT=$(git rev-parse --short "HEAD^{commit}" 2>/dev/null)
        BIN_TIMESTAMP=$(date +'%Y%m%d')
        SOURCE_GIT_TAG_NO_V=$(echo ${SOURCE_GIT_TAG} | sed 's/^v//')
        MAJOR=$(echo ${SOURCE_GIT_TAG_NO_V} | awk -F'[._~-]' '{print $1}')
        MINOR=$(echo ${SOURCE_GIT_TAG_NO_V} | awk -F'[._~-]' '{print $2}')
        PATCH=$(echo ${SOURCE_GIT_TAG_NO_V} | awk -F'[._~-]' '{print $3}')

        export GO_LDFLAGS="-X %{goipath}/pkg/version.majorFromGit=${MAJOR} -X %{goipath}/pkg/version.minorFromGit=${MINOR} -X %{goipath}/pkg/version.patchFromGit=${PATCH} -X %{goipath}/pkg/version.versionFromGit=${SOURCE_GIT_TAG} -X %{goipath}/pkg/version.commitFromGit=${SOURCE_GIT_COMMIT} -X %{goipath}/pkg/version.gitTreeState=${SOURCE_GIT_TREE_STATE} -X %{goipath}/pkg/version.buildDate=${BIN_TIMESTAMP}"

        # LDFLAGS is deprecrated in Fedora, but appears to be the variable still used in RHEL
        %if "%{dist_vendor}" != "Fedora"
            export LDFLAGS="${GO_LDFLAGS}"
        %endif

        for cmd in cmd/* ; do
          %gobuild -o %{gobuilddir}/bin/$(basename $cmd) %{goipath}/$cmd
        done
    %else
        # if this is a buggy version of go we need to set GOPROXY as workaround
        # see https://github.com/golang/go/issues/61928
        GOENVFILE=$(go env GOROOT)/go.env
        if [[ ! -f "{$GOENVFILE}" ]]; then
            export GOPROXY='https://proxy.golang.org,direct'
        fi
        make build-cli build-agent
    %endif

%install
    %if "%{build_mode}" == "upstream"
         install -m 0755 -vd                     %{buildroot}%{_bindir}
         install -m 0755 -vp %{gobuilddir}/bin/* %{buildroot}%{_bindir}/


         %{buildroot}%{_bindir}/flightctl completion bash > flightctl-completion.bash
         install -Dpm 0644 flightctl-completion.bash %{buildroot}/%{_datadir}/bash-completion/completions/flightctl-completion.bash
         %{buildroot}%{_bindir}/flightctl completion fish > flightctl-completion.fish
         install -Dpm 0644 flightctl-completion.fish %{buildroot}/%{_datadir}/fish/vendor_completions.d/flightctl-completion.fish
         %{buildroot}%{_bindir}/flightctl completion zsh > _flightctl-completion
         install -Dpm 0644 _flightctl-completion %{buildroot}/%{_datadir}/zsh/site-functions/_flightctl-completion

         rm -f licenses.list

         find -type f -name LICENSE -or -name License | while read LICENSE_FILE; do
             install -Dv -m0644 "${LICENSE_FILE}" "%{buildroot}%{_datadir}/licenses/%{NAME}/${LICENSE_FILE}"
             echo "%{_datadir}/licenses/%{NAME}/${LICENSE_FILE}" >> licenses.list
         done

         cat licenses.list

         mkdir -vp "%{buildroot}%{_docdir}/%{NAME}"

         for DOC in docs examples .markdownlint-cli2.yaml README.md; do
             cp -vr "${DOC}" "%{buildroot}%{_docdir}/%{NAME}/${DOC}"
         done

         mkdir -vp %{buildroot}/%{_sharedstatedir}/flightctl \
                   %{buildroot}/usr/lib/systemd/system

         install -Dm 0755 packaging/greenboot/flightctl-agent-running-check.sh %{buildroot}/usr/lib/greenboot/check/required.d/20_check_flightctl_agent.sh
         cp packaging/systemd/flightctl-agent.service %{buildroot}/usr/lib/systemd/system
         cp packaging/must-gather/flightctl-must-gather %{buildroot}/%{_bindir}
    %else
        mkdir -p %{buildroot}/usr/bin
        cp bin/flightctl %{buildroot}/usr/bin
        mkdir -p %{buildroot}/usr/lib/systemd/system
        mkdir -p %{buildroot}/%{_sharedstatedir}/flightctl
        mkdir -p %{buildroot}/usr/lib/flightctl/hooks.d/{afterupdating,beforeupdating,afterrebooting,beforerebooting}
        mkdir -p %{buildroot}/usr/lib/greenboot/check/required.d
        install -m 0755 packaging/greenboot/flightctl-agent-running-check.sh %{buildroot}/usr/lib/greenboot/check/required.d/20_check_flightctl_agent.sh
        cp bin/flightctl-agent %{buildroot}/usr/bin
        cp packaging/must-gather/flightctl-must-gather %{buildroot}/usr/bin
        cp packaging/hooks.d/afterupdating/00-default.yaml %{buildroot}/usr/lib/flightctl/hooks.d/afterupdating
        cp packaging/systemd/flightctl-agent.service %{buildroot}/usr/lib/systemd/system
        bin/flightctl completion bash > flightctl-completion.bash
        install -Dpm 0644 flightctl-completion.bash -t %{buildroot}/%{_datadir}/bash-completion/completions
        bin/flightctl completion fish > flightctl-completion.fish
        install -Dpm 0644 flightctl-completion.fish -t %{buildroot}/%{_datadir}/fish/
        bin/flightctl completion zsh > _flightctl-completion
        install -Dpm 0644 _flightctl-completion -t %{buildroot}/%{_datadir}/zsh/site-functions/

        rm -f licenses.list

        find -type f -name LICENSE -or -name License | while read LICENSE_FILE; do
            echo "%{_datadir}/licenses/%{NAME}/${LICENSE_FILE}" >> licenses.list
        done
        mkdir -vp "%{buildroot}%{_datadir}/licenses/%{NAME}"
        cp LICENSE "%{buildroot}%{_datadir}/licenses/%{NAME}"

        mkdir -vp "%{buildroot}%{_docdir}/%{NAME}"

        for DOC in docs examples .markdownlint-cli2.yaml README.md; do
            cp -vr "${DOC}" "%{buildroot}%{_docdir}/%{NAME}/${DOC}"
        done
    %endif

%check
    %if "%{build_mode}" == "upstream"
        export GOPATH="%{_builddir}/%{name}-%{version}/_build"
        cd "_build/src/github.com/flightctl/%{name}"
        for d in $(go list %{?exp} ./... | grep -v 'cmd\|scripts\|internal/tpm\|e2e/agent\|e2e/cli\|integration/agent\|integration/store\|device/console\|integration/tasks' ); do
            go test %{?exp} ${d}
        done
    %else
        %{buildroot}%{_bindir}/flightctl-agent version
    %endif

# File listings
# No %files section for the main package, so it won't be built

%files cli -f licenses.list
    %{_bindir}/flightctl
    %license LICENSE
    %if "%{build_mode}" == "upstream"
        %license vendor/modules.txt
    %endif
    %{_datadir}/bash-completion/completions/flightctl-completion.bash
    %{_datadir}/fish/vendor_completions.d/flightctl-completion.fish
    %{_datadir}/zsh/site-functions/_flightctl-completion
    %{_docdir}/%{NAME}/*
    %{_docdir}/%{NAME}/.markdownlint-cli2.yaml

%files agent -f licenses.list
    %license LICENSE
    %{_bindir}/flightctl-agent
    %{_bindir}/flightctl-must-gather
    /usr/lib/flightctl/hooks.d/afterupdating/00-default.yaml
    /usr/lib/systemd/system/flightctl-agent.service
    %{_sharedstatedir}/flightctl
    /usr/lib/greenboot/check/required.d/20_check_flightctl_agent.sh
    %{_docdir}/%{NAME}/*
    %{_docdir}/%{NAME}/.markdownlint-cli2.yaml


%changelog
* Thu Jan 2 2025 Packit <hello@packit.dev> - 0.3.0-1.20250102151221086407.pr735.193.gc4e86a1
- EDM-885: Sync spec with productization (Gregory Shilin)
- EDM-885: Sync spec with productization (Gregory Shilin)
- EDM-885: Sync spec with productization (Gregory Shilin)
- EDM-781: Field selectors tests (Eldar Weiss)
- EDM-390: Set enrolled devices to 'Enrolled' lifecycle status (Lily Sturmann)
- EDM-954: Use approved RabbitMQ (Avishay Traeger)
- EDM-819: Remove any unnecessary helm for the gRPC endpoint (Miguel Angel Ajo)
- EDM-818: gRPC and agent HTTPS under the same endpoint (Miguel Angel Ajo)
- EDM-955: Use approved Redis (Avishay Traeger)
- NO-ISSUE: Collect logs in smoke/e2e github workflows (Avishay Traeger)
- EDM-922: Fine-tune HTTP server settings (Avishay Traeger)
- EDM-953: Use approved Postgres (Avishay Traeger)
- NO-ISSUE: omit grpc from make generate (Sam Batschelet)
- EDM-649: Update CSR error codes to be more expressive and accurate (Lily Sturmann)
- NO-ISSUE: reduce scope of bootc API (Sam Batschelet)
- Revert "NO-ISSUE: helm: unblock CI pin rabbitmq" (Sam Batschelet)
- EDM-390: Add decommission CLI command (Lily Sturmann)
- EDM-390: Small fix for decommission target type (Lily Sturmann)
- NO-ISSUE: troubleshooting: add basic agent debug (Sam Batschelet)
- EDM-948: Allow specifying Namespace for k8s RBAC checks (#736) (Rastislav Wagner)
- EDM-870: flightctl console cli on web socket (#731) (Miguel Angel Ajo Pelayo)
- MGMT-18492: Add logical support for reboot hooks (Sam Batschelet)
- EDM-390: Add lifecyclestatus to devicestatus (Lily Sturmann)
- NO-ISSUE: libvirt-dev should always be installed (Miguel Angel Ajo)
- NO-ISSUE: Implement Go tools pattern (Frank A. Zdarsky)
- NO-ISSUE: Update GH runner (Frank A. Zdarsky)
- EDM-390: Add decommissioning field to rendered device spec (Lily Sturmann)
- NO-ISSUE: packaging: add flightctl agent version to must-gather (Sam Batschelet)
- NO-ISSUE: metrics: register flightctl_api_errors_client_total (Sam Batschelet)
- EDM-391: Add 'decommissioning' Condition and Reasons for device (Lily Sturmann)
- NO-ISSUE: Use a password for accessing Valkey (Avishay Traeger)
- NO-ISSUE: Move key-value code out of tasks package (Avishay Traeger)
- EDM-854: Use go-templates for fleet template parameters (Avishay Traeger)
- NO-ISSUE: helm: unblock CI pin rabbitmq (Sam Batschelet)
- EDM-879: improve service generated app status (Sam Batschelet)
- EDM-886: correctly fmt service side app status (Sam Batschelet)
- EDM-526: ensure errors reported by os.exec are properly handeled (Sam Batschelet)
- EDM-870: ws console wiring (Miguel Angel Ajo)
- EDM-870: Move console request/connection to websocket (Miguel Angel Ajo)
- EDN-860: Add permission checks for all API endpoints (Ori Amizur)
- NO-ISSUE: Fix OpenAPI descriptions (Frank A. Zdarsky)
- EDM-848: revert tzdata host mount (#690) (Sam Batschelet)
- EDM-876: Fix helm template error (Celia Amador)
- EDM-788: Add init script for PostgreSQL to set master user as superuser (#679) (Assaf Albo)
- EDM-838: Update ACM registration doc (Frank A. Zdarsky)
- NO-ISSUE: redeploy kind pods for development (Miguel Angel Ajo)
- NO-ISSUE: move ownership od status exporters to managers (Sam Batschelet)
- EDM-853: spec/queue: ensure item requeue backoff only if tried  (#696) (Sam Batschelet)
- EDM-642: Remove sorting mechanism for TP phase (#701) (Assaf Albo)
- MGMT-17672: Document device statuses (Frank A. Zdarsky)
- EDM-417: Get renderedVersion from DB (Frank A. Zdarsky)
- EDM-417: Update unit-tests for server-side status (Frank A. Zdarsky)
- EDM-417: Do not report retries, fail faster (Frank A. Zdarsky)
- EDM-417: Compute service-side status (Frank A. Zdarsky)
- EDM-649: Add specific error message when CSR approval fails (Lily Sturmann)
- EDM-649: Remove extraneous CN len check (Lily Sturmann)
- EDM-842: transient networking error should not mark agent degraded (Sam Batschelet)
- EDM-772: Move UI chart to main repo (#673) (Rastislav Wagner)
- NO-ISSUE: agent: ensure bootstrap status (Sam Batschelet)
- EDM-761: Document lifecycle hooks (Frank A. Zdarsky)
- EDM-764: Update Hooks API implementation (Frank A. Zdarsky)
- EDM-764: Update Hooks API (Frank A. Zdarsky)
- EDM-659: agent/fileio: ensure ignition gid, uid and mode (Sam Batschelet)
- NO-ISSUE: Pass internalOpenShiftApiUrl from values to api-config CM (#680) (Rastislav Wagner)
- EDM-424: add update policy api and validation (Sam Batschelet)
- EDM-420: Increase DB parameters (#669) (Gregory Shilin)
- NO-ISSUE: Append UI port to valid redirect URIs (rawagner)
- EDM-390: Add decommissioning route to device API (Lily Sturmann)
- EDM-417: Centralize API consts (Frank A. Zdarsky)
- NO-ISSUE: add command to prepare e2e test (sserafin)
- EDM-621: Fix microshift registration docs (Avishay Traeger)
- EDM-767: Actually update ignition mount path (Avishay Traeger)
- EDM-708: Reduce git access (Avishay Traeger)
- NO-ISSUE: Remove duplicated code in git helpers (Avishay Traeger)
- MGMT-17672: device: align update status phases (Sam Batschelet)
- Add developer documentation for field-selector (#658) (Assaf Albo)
- EDM-681: Allow field selector when 'summaryOnly' is enabled (#656) (Assaf Albo)
- EDM-704: Automate test 75991 (sserafin)
- EDM-695: typo fix (Gregory Shilin)
- EDM-695: typo fix (Gregory Shilin)
- EDM-695: typo fix (Gregory Shilin)
- EDM-695: typo fix (Gregory Shilin)
- EDM-695: created a document with instructions to run devicesimulator (Gregory Shilin)
- EDM-695: Logger verbosity flag was added (-v) (#651) (Gregory Shilin)
- EDM-423: Fix lexer state for nested exists/notexists selectors (#655) (Assaf Albo)
- EDM-423: Adapt K8s lexer to support RHS equality symbol (asafbss)
- EDM-423: Modify k8s containment operator; remove support for 'in' and 'notin' on JSON fields (asafbss)
- EDM-423: Remove selector casting; implement a whitelist of selectors per resource for stricter filtering (asafbss)
- EDM-423: Move alias back to label in API while keeping it as a field in device model (asafbss)
- EDM-423: Add documentation for Field Selectors (asafbss)
- EDM-666: Updated selector names to match the documented API (asafbss)
- EDM-664: Added selector support for Spec field in field-selector (asafbss)
- EDM-423: Introduce Alias Field for Device Metadata (asafbss)
- EDM-423: Add more Kubernetes operators to enhance selectors for fields (asafbss)
- EDM-423: Kubernetes vanilla label selector (asafbss)
- NO-ISSUE: Fix linter and spellchecker issues (Frank A. Zdarsky)
- NO-ISSUE: Fix path-filter (Frank A. Zdarsky)
- EDM-694: Add Valkey key value store service (Ricardo Noriega)
- EDM-694: Use primary IP and nip.io domains (Ricardo Noriega)
- EDM-694: Add FlightCtl Network to slice (Ricardo Noriega)
- EDM-694: Add Makefile Quadlet targets for deploy and clean (Ricardo Noriega)
- EDM-694: Initial FlightCtl deployment with Quadlets (Ricardo Noriega)
- NO-ISSUE: Update config storage API (Avishay Traeger)
- EDM-578: Remove templateversion populate task (Avishay Traeger)
- EDM-582: Freeze HTTP configurations (Avishay Traeger)
- EDM-583: Freeze k8s secret configurations (Avishay Traeger)
- EDM-581: Freeze git configurations (Avishay Traeger)
- NO-ISSUE: Remove device's TV annotation if removed from fleet (Avishay Traeger)
- EDM-580: Deploy Valkey for frozen configurations (Avishay Traeger)
- EDM-578: Separate fleet validate and device render logic (Avishay Traeger)
- EDM-420: Enable to run devicesimulator on multiple hosts (#650) (Gregory Shilin)
- NO-ISSUE: Improve UpdateDeviceWithRetries to handle all errors (Miguel Angel Ajo)
- EDM-446: device/fileio: ensure readable paths in PathExists (Sam Batschelet)
- EDM-656: agent: clarify completed status (Sam Batschelet)
- EDM-656: agent: ensure Shutdown is threadsafe (Sam Batschelet)
- EDM-656: agent/device improve systemd status reporting (Sam Batschelet)
- NO-ISSUE: console: simplify exec test case (Sam Batschelet)
- EDM-673: Enable exposing APIs via Gateway (rawagner)
- EDM-650: Load k8s ca.crt when deployed on k8s (rawagner)

* Mon Nov 4 2024 Miguel Angel Ajo <majopela@redhat.com> - 0.3.0-1
- Move the Release field to -1 so we avoid auto generating packages
  with -5 all the time.
* Wed Aug 21 2024 Sam Batschelet <sbatsche@redhat.com> - 0.0.1-5
- Add must-gather script to provide a simple mechanism to collect agent debug
* Wed Aug 7 2024 Sam Batschelet <sbatsche@redhat.com> - 0.0.1-4
- Add basic greenboot support for failed flightctl-agent service
* Wed Mar 13 2024 Ricardo Noriega <rnoriega@redhat.com> - 0.0.1-3
- New specfile for both CLI and agent packages
